########
# Author: ajb@sanger.ac.uk
# Maintainer: $Author: gq1 $
# Created: 2008/10/14
# Id: $Id: colourmap.pm 3512 2008-11-18 14:21:33Z gq1 $
#

package npg_common::graphics::colourmap;
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};

sub new {
    my ($class) = @_;

    my $self = {
                'black'           => '000000',
                'blue'            => '0000ff',
                'cyan'            => '00ffff',
                'darkblue'        => '00008b',
                'darkcyan'        => '008b8b',
                'darkgreen'       => '006400',
                'darkgrey'        => 'a9a9a9',
                'darkmagenta'     => '8b008b',
                'darkorange'      => 'ff8c00',
                'darkred'         => '8b0000',
                'gold'            => 'ffd700',
                'green'           => '00ff00',
                'grey'            => 'bebebe',
                'ivory'           => 'fffff0',
                'lightblue'       => 'add8e6',
                'lightgreen'      => '90ee90',
                'lightgrey'       => 'd3d3d3',
                'lightpink'       => 'ffb6c1',
                'lightyellow'     => 'ffffe0',
                'magenta'         => 'ff00ff',
                'maroon'          => 'b03060',
                'mediumblue'      => '0000cd',
                'mediumpurple'    => '9370db',
                'navy'            => '000080',
                'orange'          => 'ffa500',
                'palegreen'       => '98fb98',
                'pink'            => 'ffc0cb',
                'purple'          => 'a020f0',
                'red'             => 'ff0000',
                'royalblue'       => '4169e1',
                'skyblue'         => '87ceeb',
                'turquoise'       => '40e0d0',
                'violet'          => 'ee82ee',
                'white'           => 'ffffff',
                'yellow'          => 'ffff00',
	       };

    bless($self, $class);
    return $self;
}
sub hex_by_name {
    my ($self, $name) = @_;
    return $self->{$name} || ( $name =~ /^#?([0-9a-fA-F]{6})$/ ? $1 : 'ff0000' ); 
}

sub rgb_by_name {
  my ($self, $name) = @_;
  $name = lc($name);
  my $hex=$self->{$name};
  $hex=$name if (! $hex) and $name=~/^[0-9a-f]{6}$/;
  unless( $hex ) {
    warn "Unknown colour name $name";
    return (hex(0), hex(0), hex(0));
  }
  return $self->rgb_by_hex($hex);
}

sub add_rgb {
    my ($self, $triple_ref) = @_;
    return $self->add_hex( $self->hex_by_rgb($triple_ref) );
}

sub add_hex {
    my ($self, $hex) = @_;
    $self->{$hex} = $hex;
    return $hex;
}

sub hex_by_rgb {
    my ($self, $triple_ref) = @_;
    return sprintf("%02x%02x%02x", @{$triple_ref});
}

sub rgb_by_hex {
    my ($self, $hex) = @_;

    unless($hex){
      warn "Cannot map hex colour in colourmap\n";
      return(hex(0), hex(0), hex(0));
    }
    my ($hred, $hgreen, $hblue) = unpack("A2A2A2", $hex);
    return (hex($hred), hex($hgreen), hex($hblue));
}

sub build_linear_gradient {
  my ($self, $grades_total, $start, @colours) = @_;

  my @finished_gradient = ();

  #########
  # deal with a single arrayref argument if supplied
  #
  if(scalar @colours == 0 && ref($start) eq "ARRAY") {
    @colours = @{$start};
    $start   = shift @colours;
  }

  my $tgrades  = scalar @colours || 1;
  my $sgrades  = $grades_total / $tgrades;

  while(my $end = shift @colours) {
    my ($sr, $sg, $sb) = $self->rgb_by_name($start);
    my ($er, $eg, $eb) = $self->rgb_by_name($end);
    my $dr             = ($er - $sr) / $sgrades;
    my $dg             = ($eg - $sg) / $sgrades;
    my $db             = ($eb - $sb) / $sgrades;
    my ($r, $g, $b)    = ($sr, $sg, $sb);
    
    for (my $i = 0; $i < $sgrades; $i++) {
      push @finished_gradient, $self->add_rgb([$r, $g, $b]);
      $r += $dr;
      $g += $dg;
      $b += $db;
    }

    $start = $end;
  }

  #########
  # work around for rounding error (incorrect number of colours returned under certain conditions)
  #
  pop @finished_gradient if(scalar @finished_gradient > $grades_total);

  return @finished_gradient;
}

1;
__END__

=head1 NAME

npg_common::graphics::colourmap

=head1 VERSION

$LastChangedRevision: 3512 $

=head1 SYNOPSIS

  my $oColourMap = npg_common::graphics::colourmap->new();

=head1 DESCRIPTION

Object to provide required functionality to be used instead of Sanger::Graphics:ColourMap

=head1 SUBROUTINES/METHODS

=head2 new - constructor to create a colourmap object
=head2 hex_by_name - gets the hex value by name
=head2 rgb_by_name - gets the rgb value by name
=head2 add_rgb
=head2 add_hex
=head2 hex_by_rgb - gets the hex value from the rgb value
=head2 rgb_by_hex - gets the rgb value from the hex value
=head2 build_linear_gradient - builds a linear gradient of colours between two give colours (with optional additionals inbetween) over a number of selected steps

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Carp
English

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
